

#ifndef _treeview_h_ /* Fri Jan 24 11:53:31 1997 */
#define _treeview_h_

// A TreeView is a simple visual object that displays a tree in graphical
// form. The model for a TreeView is the UI_TreeViewModel object (see
// ui/treevmdl.h), which represents a tree of pointers to instances of
// (classes derived from) CL_Object. The TreeView class
// assumes that it can use the AsString method to obtain the text of
// each node. Also, neither the TreeView nor its model take responsibility
// for the instances of the UI_TreeViewNodeContent class; those instances
// must be managed by the application program.
//
// Every node in the model is required to have a distinct, positive
// integer nodeId. This id will be used to notify the TreeView of events
// on its nodes.
//
// The TreeView supports three kinds of events:
//    - NodeCollapsed, sent when the user collapses a node
//    - NodeExpanded, sent when the user expands a node
//    - Select, sent when the user selects a node
// When one of these events occurs, the Controller creates a
// UI_TreeNodeEvent of the appropriate type and sends it to this
// TreeView. The TreeView defines event handler methods NodeCollapsed and
// NodeExpanded, which will (respectively) be called when these events are
// received. The default implementations of these methods simply return
// FALSE. Derived classes may override these methods.
//
// CAVEAT: This class is currently ONLY available under OS/2.


#include "base/tree.h"
#include "base/integer.h"

#include "ui/simple.h"
#include "ui/treevmdl.h"


class YACL_UI UI_TreeView: public UI_SimpleVObject {

public:
    UI_TreeView (UI_VisualObject* parent, const UI_Rectangle& shape,
                 UI_ViewID id);

    UI_TreeView (UI_VisualObject* parent, const UI_Rectangle& shape,
                 UI_ViewID id, UI_TreeViewModel* model);


    CL_Integer& Selection() {return _selectedNode;};
    // Return s reference to an Integer containing the node id of the
    // currently selected node. The integer contains -1 if no node has
    // been selected.
    
    void SetIndentDepth (short nPixels);
    // Set the indentation depth to the specified number of pixels.

    void SetSeparation  (short nPixels);
    // Set the separation between consective lines in the tree display to
    // the specified number of pixels.

    bool CollapseNode   (CL_TreeNodeLabel  nodeId);
    // Collapse the specified node under application program control, if
    // it isn't already collapsed.

    bool ExpandNode     (CL_TreeNodeLabel  nodeId);
    // Expand the specified node under application program control, if
    // it isn't already expanded.

    bool IsExpanded     (CL_TreeNodeLabel  nodeId);
    // Return TRUE if the node with the specified id is currently
    // expanded, and FALSE otherwise.
    
    UI_WindowClass WindowClass () const;

    const char* ClassName () const {return "UI_TreeView";};

protected:
    ~UI_TreeView ();
    // Protected destructor, to prevent accidental delete by user program.
    
    virtual bool NodeCollapsed (long nodeId);
    // Event handler called (by the controller) when the user collapses
    // the specified node.

    virtual bool NodeExpanded  (long nodeId);
    // Event handler called (by the controller) when the user expands
    // the specified node.

    bool MakeVisualElement ();

    void _AddChildren (const CL_ObjectSequence& siblings, void*);

    bool HandleEvent (UI_Event* e);

    void _DoDestroy (CL_ObjectPtrTreeNode* );
    // [For YACL internal use only]
    
    CL_Integer _selectedNode;
    
    short _separation;
    short _indent;

    // UI_TreeViewEvent is declared a friend so that its Dispatch method
    // may call this object's event handlers.
    friend class YACL_UI UI_TreeViewEvent;


public:
#if defined(__OS2__)
    long NodeIdFromMessage (NativeEventStruct& evt);
    // [OS/2-specific, for internal use only]

    CL_IntIntMap _recordCoreMap; // Used only under OS/2 
#endif
    
    void ModelHasAddedNode (CL_ObjectPtrTreeNode* newNode);
    // [YACL internal use only] Called by our model to let us know that it
    // has a new node has been added to the model tree.

    void ModelNodeContentChanged (CL_ObjectPtrTreeNode* node);
    // [YACL internal use only] Called by our model to let us know that
    // the specified node has new contents.
    
    void ModelHasNewRoot ();
    // [YACL internal use only] Called by our model to let us know that it
    // has a new root.

    void ModelWillDestroySubtree (CL_ObjectPtrTreeNode* node);
    // [YACL internal use only] Called by our model to let us know that it
    // is about to destroy the subtree rooted at the given node.
    
    
};


    
#endif /* _treeview_h_ */
